@@ -38,6 +38,7 @@
 #include "wpa.h"
 #include "radius.h"
 #include "ieee802_11.h"
+#include "common.h"
 
 struct bsd_driver_data {
 	struct driver_ops ops;			/* base class */
@@ -333,9 +334,12 @@ bsd_del_key(void *priv, unsigned char *addr, int key_idx)
 		__func__, ether_sprintf(addr), key_idx);
 
 	memset(&wk, 0, sizeof(wk));
-	wk.idk_keyix = key_idx;
-	if (addr != NULL)
+	if (addr != NULL) {
 		memcpy(wk.idk_macaddr, addr, IEEE80211_ADDR_LEN);
+		wk.idk_keyix = (u_int8_t) IEEE80211_KEYIX_NONE;	/* XXX */
+	} else {
+		wk.idk_keyix = key_idx;
+	}
 
 	return set80211var(priv, IEEE80211_IOC_DELKEY, &wk, sizeof(wk));
 }
@@ -412,10 +416,25 @@ bsd_get_seqnum(void *priv, u8 *addr, int idx, u8 *seq)
 	if (get80211var(drv, IEEE80211_IOC_WPAKEY, &wk, sizeof(wk)) < 0) {
 		printf("Failed to get encryption.\n");
 		return -1;
-	} else {
-		memcpy(seq, &wk.ik_keytsc, sizeof(wk.ik_keytsc));
-		return 0;
 	}
+
+#ifdef WORDS_BIGENDIAN
+	{
+		/*
+		 * wk.ik_keytsc is in host byte order (big endian), need to
+		 * swap it to match with the byte order used in WPA.
+		 */
+		int i;
+		u8 tmp[WPA_KEY_RSC_LEN];
+		memcpy(tmp, &wk.ik_keytsc, sizeof(wk.ik_keytsc));
+		for (i = 0; i < WPA_KEY_RSC_LEN; i++) {
+			seq[i] = tmp[WPA_KEY_RSC_LEN - i - 1];
+		}
+	}
+#else /* WORDS_BIGENDIAN */
+	memcpy(seq, &wk.ik_keytsc, sizeof(wk.ik_keytsc));
+#endif /* WORDS_BIGENDIAN */
+	return 0;
 }
 
 
@@ -574,12 +593,14 @@ bsd_new_sta(struct bsd_driver_data *drv, u8 addr[IEEE80211_ADDR_LEN])
 	 */
 	new_assoc = (sta->flags & WLAN_STA_ASSOC) == 0;
 	sta->flags |= WLAN_STA_ASSOC;
-	if (conf->wpa)
-		wpa_sm_event(hapd, sta, WPA_ASSOC);
-	if (new_assoc)
+	if (new_assoc) {
+		if (conf->wpa)
+			wpa_sm_event(hapd, sta, WPA_ASSOC);
 		hostapd_new_assoc_sta(hapd, sta);
-	else if (conf->wpa)
-		wpa_sm_event(hapd, sta, WPA_REAUTH);
+	} else {
+		if (conf->wpa)
+			wpa_sm_event(hapd, sta, WPA_REAUTH);
+	}
 	ieee802_1x_notify_port_enabled(sta->eapol_sm, 1);
 	return 0;
 }
@@ -627,6 +648,9 @@ bsd_wireless_event_receive(int sock, void *ctx, void *sock_ctx)
 			bsd_del_sta(drv, leave->iev_addr);
 			break;
 		case RTM_IEEE80211_JOIN:
+#ifdef RTM_IEEE80211_REJOIN
+		case RTM_IEEE80211_REJOIN:
+#endif
 			join = (struct ieee80211_join_event *) &ifan[1];
 			bsd_new_sta(drv, join->iev_addr);
 			break;
