@@ -45,6 +45,7 @@
 #include "radius.h"
 #include "ieee802_11.h"
 #include "accounting.h"
+#include "common.h"
 
 
 struct madwifi_driver_data {
@@ -359,9 +360,12 @@ madwifi_del_key(void *priv, unsigned char *addr, int key_idx)
 		__func__, ether_sprintf(addr), key_idx);
 
 	memset(&wk, 0, sizeof(wk));
-	wk.idk_keyix = key_idx;
-	if (addr != NULL)
+	if (addr != NULL) {
 		memcpy(wk.idk_macaddr, addr, IEEE80211_ADDR_LEN);
+		wk.idk_keyix = (u8) IEEE80211_KEYIX_NONE;
+	} else {
+		wk.idk_keyix = key_idx;
+	}
 
 	return set80211priv(priv, IEEE80211_IOCTL_DELKEY, &wk, sizeof(wk));
 }
@@ -396,7 +400,8 @@ madwifi_set_key(void *priv, const char *alg,
 	}
 
 	if (key_len > sizeof(wk.ik_keydata)) {
-		printf("%s: key length %d too big\n", __func__, key_len);
+		printf("%s: key length %lu too big\n", __func__,
+		       (unsigned long) key_len);
 		return -3;
 	}
 
@@ -438,10 +443,25 @@ madwifi_get_seqnum(void *priv, u8 *addr, int idx, u8 *seq)
 	if (set80211priv(priv, IEEE80211_IOCTL_GETKEY, &wk, sizeof(wk))) {
 		printf("Failed to get encryption.\n");
 		return -1;
-	} else {
-		memcpy(seq, &wk.ik_keytsc, sizeof(wk.ik_keytsc));
-		return 0;
 	}
+
+#ifdef WORDS_BIGENDIAN
+	{
+		/*
+		 * wk.ik_keytsc is in host byte order (big endian), need to
+		 * swap it to match with the byte order used in WPA.
+		 */
+		int i;
+		u8 tmp[WPA_KEY_RSC_LEN];
+		memcpy(tmp, &wk.ik_keytsc, sizeof(wk.ik_keytsc));
+		for (i = 0; i < WPA_KEY_RSC_LEN; i++) {
+			seq[i] = tmp[WPA_KEY_RSC_LEN - i - 1];
+		}
+	}
+#else /* WORDS_BIGENDIAN */
+	memcpy(seq, &wk.ik_keytsc, sizeof(wk.ik_keytsc));
+#endif /* WORDS_BIGENDIAN */
+	return 0;
 }
 
 
@@ -874,7 +894,7 @@ madwifi_send_eapol(void *priv, u8 *addr, u8 *data, size_t data_len, int encrypt)
 		bp = malloc(len);
 		if (bp == NULL) {
 			printf("EAPOL frame discarded, cannot malloc temp "
-				"buffer of size %u!\n", len);
+			       "buffer of size %lu!\n", (unsigned long) len);
 			return -1;
 		}
 	}
